@@ -38,9 +38,10 @@
 #include <pdal/PointView.hpp>
 #include <pdal/StageFactory.hpp>
 #include <pdal/Polygon.hpp>
-#include <pdal/pdal_macros.hpp>
 #include <pdal/util/ProgramArgs.hpp>
-#include <filters/private/crop/Point.hpp>
+
+#include "private/Point.hpp"
+#include "private/pnp/GridPnp.hpp"
 
 #include <sstream>
 #include <cstdarg>
@@ -48,34 +49,56 @@
 namespace pdal
 {
 
-static PluginInfo const s_info = PluginInfo(
+static StaticPluginInfo const s_info
+{
     "filters.crop",
     "Filter points inside or outside a bounding box or a polygon",
-    "http://pdal.io/stages/filters.crop.html" );
+    "http://pdal.io/stages/filters.crop.html"
+};
 
-CREATE_STATIC_PLUGIN(1, 0, CropFilter, Filter, s_info)
+CREATE_STATIC_STAGE(CropFilter, s_info)
+
+struct CropArgs
+{
+    bool m_cropOutside;
+    SpatialReference m_assignedSrs;
+    std::vector<Bounds> m_bounds;
+    std::vector<filter::Point> m_centers;
+    double m_distance;
+    std::vector<Polygon> m_polys;
+};
+
+CropFilter::ViewGeom::ViewGeom(const Polygon& poly) : m_poly(poly)
+{}
+
+CropFilter::ViewGeom::ViewGeom(ViewGeom&& vg) :
+    m_poly(std::move(vg.m_poly)), m_gridPnps(std::move(vg.m_gridPnps))
+{}
 
 std::string CropFilter::getName() const { return s_info.name; }
 
-CropFilter::CropFilter() : m_cropOutside(false)
+CropFilter::CropFilter() : m_args(new CropArgs)
 {}
 
+
 CropFilter::~CropFilter()
 {}
 
+
 void CropFilter::addArgs(ProgramArgs& args)
 {
     args.add("outside", "Whether we keep points inside or outside of the "
-        "bounding region", m_cropOutside);
-    args.add("a_srs", "Spatial reference for bounding region", m_assignedSrs);
-    args.add("bounds", "Point box for cropped points", m_bounds);
+        "bounding region", m_args->m_cropOutside);
+    args.add("a_srs", "Spatial reference for bounding region",
+        m_args->m_assignedSrs);
+    args.add("bounds", "Point box for cropped points", m_args->m_bounds);
     args.add("point", "Center of circular/spherical crop region.  Use with "
-        "'distance'.", m_centers).setErrorText("Invalid point specification.  "
-            "Must be valid GeoJSON/WKT. "
-            "Ex: \"(1.00, 1.00)\" or \"(1.00, 1.00, 1.00)\"");
+        "'distance'.", m_args->m_centers).
+        setErrorText("Invalid point specification.  Must be valid "
+            "GeoJSON/WKT. Ex: \"(1.00, 1.00)\" or \"(1.00, 1.00, 1.00)\"");
     args.add("distance", "Crop with this distance from 2D or 3D 'point'",
-        m_distance);
-    args.add("polygon", "Bounding polying for cropped points", m_polys).
+        m_args->m_distance);
+    args.add("polygon", "Bounding polying for cropped points", m_args->m_polys).
         setErrorText("Invalid polygon specification.  "
             "Must be valid GeoJSON/WKT");
 }
@@ -84,41 +107,53 @@ void CropFilter::addArgs(ProgramArgs& args)
 void CropFilter::initialize()
 {
     // Set geometry from polygons.
-    if (m_polys.size())
+    if (m_args->m_polys.size())
     {
         m_geoms.clear();
-        for (Polygon& poly : m_polys)
+        for (Polygon& poly : m_args->m_polys)
         {
             // Throws if invalid.
             poly.valid();
-            m_geoms.push_back(poly);
+            m_geoms.emplace_back(poly);
         }
     }
-    m_distance2 = m_distance * m_distance;
+
+    m_boxes.clear();
+    for (auto& bound : m_args->m_bounds)
+        m_boxes.push_back(bound.to2d());
+
+    m_distance2 = m_args->m_distance * m_args->m_distance;
 }
 
 
 void CropFilter::ready(PointTableRef table)
 {
     // If the user didn't provide an SRS, take one from the table.
-    if (m_assignedSrs.empty())
-        m_assignedSrs = table.anySpatialReference();
+    if (m_args->m_assignedSrs.empty())
+    {
+        m_args->m_assignedSrs = table.anySpatialReference();
+        if (!table.spatialReferenceUnique())
+            log()->get(LogLevel::Warning) << "Can't determine spatial "
+                "reference for provided bounds.  Consider using 'a_srs' "
+                "option.\n";
+    }
     for (auto& geom : m_geoms)
-        geom.setSpatialReference(m_assignedSrs);
+        geom.m_poly.setSpatialReference(m_args->m_assignedSrs);
 }
 
 
 bool CropFilter::processOne(PointRef& point)
 {
-    for (auto& geom : m_geoms)
-        if (!crop(point, geom))
+    for (auto& g : m_geoms)
+        for (auto& gridPnp : g.m_gridPnps)
+            if (!crop(point, *gridPnp))
+                return false;
+
+    for (auto& box : m_boxes)
+        if (!crop(point, box))
             return false;
 
-    for (auto& box : m_bounds)
-        if (!crop(point, box.to2d()))
-            return false;
-
-    for (auto& center: m_centers)
+    for (auto& center: m_args->m_centers)
         if (!crop(point, center))
             return false;
 
@@ -134,37 +169,48 @@ void CropFilter::spatialReferenceChanged(const SpatialReference& srs)
 
 void CropFilter::transform(const SpatialReference& srs)
 {
-    // If we don't have any SRS, do nothing.
     for (auto& geom : m_geoms)
     {
         try
         {
-            geom = geom.transform(srs);
+            geom.m_poly = geom.m_poly.transform(srs);
         }
         catch (pdal_error& err)
         {
             throwError(err.what());
         }
+        geom.m_gridPnps.clear();
+        std::vector<Polygon> polys = geom.m_poly.polygons();
+        for (auto& p : polys)
+        {
+            std::unique_ptr<GridPnp> gridPnp(new GridPnp(
+                p.exteriorRing(), p.interiorRings()));
+            geom.m_gridPnps.push_back(std::move(gridPnp));
+        }
     }
 
-    if (srs.empty() && m_assignedSrs.empty())
+    // If we don't have any SRS, do nothing.
+    if (srs.empty() && m_args->m_assignedSrs.empty())
         return;
-    if (srs.empty() || m_assignedSrs.empty())
+    if (srs.empty() || m_args->m_assignedSrs.empty())
         throwError("Unable to transform crop geometry to point "
             "coordinate system.");
 
-    for (auto& box : m_bounds)
+    for (auto& box : m_boxes)
     {
-        BOX3D b3d = box.to3d();
-        gdal::reprojectBounds(b3d, m_assignedSrs.getWKT(), srs.getWKT());
-        box = b3d;
+        if (!gdal::reprojectBounds(box, m_args->m_assignedSrs.getWKT(),
+            srs.getWKT()))
+            throwError("Unable to reproject bounds.");
     }
-    for (auto& point : m_centers)
+    for (auto& point : m_args->m_centers)
     {
-        gdal::reprojectPoint(point.x, point.y, point.z,
-            m_assignedSrs.getWKT(), srs.getWKT());
+        if (!gdal::reprojectPoint(point.x, point.y, point.z,
+            m_args->m_assignedSrs.getWKT(), srs.getWKT()))
+            throwError("Unable to reproject point center.");
     }
-    m_assignedSrs = srs;
+    // Set the assigned SRS for the points/bounds to the one we've
+    // transformed to.
+    m_args->m_assignedSrs = srs;
 }
 
 
@@ -180,14 +226,14 @@ PointViewSet CropFilter::run(PointViewPtr view)
         viewSet.insert(outView);
     }
 
-    for (auto& box : m_bounds)
+    for (auto& box : m_boxes)
     {
         PointViewPtr outView = view->makeNew();
-        crop(box.to2d(), *view, *outView);
+        crop(box, *view, *outView);
         viewSet.insert(outView);
     }
 
-    for (auto& point: m_centers)
+    for (auto& point: m_args->m_centers)
     {
         PointViewPtr outView = view->makeNew();
         crop(point, *view, *outView);
@@ -204,7 +250,7 @@ bool CropFilter::crop(const PointRef& point, const BOX2D& box)
     double y = point.getFieldAs<double>(Dimension::Id::Y);
 
     // Return true if we're keeping a point.
-    return (m_cropOutside != box.contains(x, y));
+    return (m_args->m_cropOutside != box.contains(x, y));
 }
 
 
@@ -214,55 +260,60 @@ void CropFilter::crop(const BOX2D& box, PointView& input, PointView& output)
     for (PointId idx = 0; idx < input.size(); ++idx)
     {
         point.setPointId(idx);
-        if (m_cropOutside != crop(point, box))
+        if (m_args->m_cropOutside != crop(point, box))
             output.appendPoint(input, idx);
     }
 }
 
 
-bool CropFilter::crop(const PointRef& point, const Polygon& g)
-{
-    return (m_cropOutside != g.covers(point));
-}
-
-
-void CropFilter::crop(const Polygon& g, PointView& input, PointView& output)
-{
-    PointRef point = input.point(0);
-    for (PointId idx = 0; idx < input.size(); ++idx)
-    {
-        point.setPointId(idx);
-        if (crop(point, g))
-            output.appendPoint(input, idx);
-    }
-}
-
-
-bool CropFilter::crop(const PointRef& point, const cropfilter::Point& center)
+bool CropFilter::crop(const PointRef& point, GridPnp& g)
 {
     double x = point.getFieldAs<double>(Dimension::Id::X);
     double y = point.getFieldAs<double>(Dimension::Id::Y);
-    x -= center.x;
-    y -= center.y;
-    if (x > m_distance || y > m_distance)
-        return (m_cropOutside);
+    return (m_args->m_cropOutside != g.inside(x, y));
+}
+
+
+void CropFilter::crop(const ViewGeom& g, PointView& input, PointView& output)
+{
+    PointRef point = input.point(0);
+    for (auto& gridPnp : g.m_gridPnps)
+    {
+        for (PointId idx = 0; idx < input.size(); ++idx)
+        {
+            point.setPointId(idx);
+            if (crop(point, const_cast<GridPnp&>(*gridPnp)))
+                output.appendPoint(input, idx);
+        }
+    }
+}
+
+
+bool CropFilter::crop(const PointRef& point, const filter::Point& center)
+{
+    double x = point.getFieldAs<double>(Dimension::Id::X);
+    double y = point.getFieldAs<double>(Dimension::Id::Y);
+    x = std::abs(x - center.x);
+    y = std::abs(y - center.y);
+    if (x > m_args->m_distance || y > m_args->m_distance)
+        return (m_args->m_cropOutside);
 
     bool inside;
     if (center.is3d())
     {
         double z = point.getFieldAs<double>(Dimension::Id::Z);
-        z -= center.z;
-        if (z > m_distance)
-            return (m_cropOutside);
+        z = std::abs(z - center.z);
+        if (z > m_args->m_distance)
+            return (m_args->m_cropOutside);
         inside = (x * x + y * y + z * z < m_distance2);
     }
     else
         inside = (x * x + y * y < m_distance2);
-    return (m_cropOutside != inside);
+    return (m_args->m_cropOutside != inside);
 }
 
 
-void CropFilter::crop(const cropfilter::Point& center, PointView& input,
+void CropFilter::crop(const filter::Point& center, PointView& input,
     PointView& output)
 {
     PointRef point = input.point(0);
