@@ -37,11 +37,9 @@
 #include <cmath>
 #include <unordered_map>
 
-#include <pdal/pdal_export.hpp>
 #include <pdal/Options.hpp>
 #include <pdal/Polygon.hpp>
 #include <pdal/PDALUtils.hpp>
-#include <pdal/pdal_macros.hpp>
 #include <pdal/util/ProgramArgs.hpp>
 
 #include <json/json.h>
@@ -49,12 +47,14 @@
 namespace pdal
 {
 
-static PluginInfo const s_info = PluginInfo(
+static StaticPluginInfo const s_info
+{
     "filters.stats",
     "Compute statistics about each dimension (mean, min, max, etc.)",
-    "http://pdal.io/stages/filters.stats.html" );
+    "http://pdal.io/stages/filters.stats.html"
+};
 
-CREATE_STATIC_PLUGIN(1, 0, StatsFilter, Filter, s_info)
+CREATE_STATIC_STAGE(StatsFilter, s_info)
 
 std::string StatsFilter::getName() const { return s_info.name; }
 
