@@ -36,23 +36,27 @@
 
 #include <climits>
 
-#include <zlib.h>
-
 #include <pdal/Options.hpp>
-#include <pdal/pdal_export.hpp>
-#include <pdal/pdal_macros.hpp>
+#include <pdal/pdal_features.hpp>
+
+#ifdef PDAL_HAVE_ZLIB
+#include <zlib.h>
+#endif
 
 namespace pdal
 {
 
-static PluginInfo const s_info = PluginInfo(
+static StaticPluginInfo const s_info
+{
     "readers.bpf",
     "\"Binary Point Format\" (BPF) reader support. BPF is a simple \n" \
         "DoD and research format that is used by some sensor and \n" \
         "processing chains.",
-    "http://pdal.io/stages/readers.bpf.html" );
+    "http://pdal.io/stages/readers.bpf.html",
+    { "bpf" }
+};
 
-CREATE_STATIC_PLUGIN(1, 0, BpfReader, Reader, s_info)
+CREATE_STATIC_STAGE(BpfReader, s_info)
 
 std::string BpfReader::getName() const { return s_info.name; }
 
@@ -94,7 +98,7 @@ QuickInfo BpfReader::inspect()
 void BpfReader::initialize()
 {
     if (m_filename.empty())
-        throw pdal_error("Can't read BPF file without filename.");
+        throwError("Can't read BPF file without filename.");
 
     // Logfile doesn't get set until options are processed.
     m_header.setLog(log());
@@ -104,13 +108,24 @@ void BpfReader::initialize()
     // Resets the stream position in case it was already open.
     m_stream.seek(0);
     // In order to know the dimensions we must read the file header.
-    if (!m_header.read(m_stream))
-        return;
+    try
+    {
+        if (!m_header.read(m_stream))
+            return;
+        if (!m_header.readDimensions(m_stream, m_dims))
+            return;
+    }
+    catch (const BpfHeader::error& err)
+    {
+        throwError(err.what());
+    }
+#ifndef PDAL_HAVE_ZLIB
+    if (m_header.m_compression)
+        throwError("Can't read compressed BPF. PDAL wasn't built with "
+            "Zlib support.");
+#endif
 
-    if (!m_header.readDimensions(m_stream, m_dims))
-        return;
-
-    std::string code("");
+    std::string code;
     if (m_header.m_coordType == static_cast<int>(BpfCoordType::Cartesian))
        code = std::string("EPSG:4326");
     else if (m_header.m_coordType == static_cast<int>(BpfCoordType::UTM))
@@ -118,17 +133,39 @@ void BpfReader::initialize()
        uint32_t zone(abs(m_header.m_coordId));
 
        if (m_header.m_coordId > 0 && m_header.m_coordId <= 60)
-          code = std::string("EPSG:326") + (zone < 10 ? "0" : "") + Utils::toString(zone);
+          code = std::string("EPSG:326");
        else if (m_header.m_coordId < 0 && m_header.m_coordId >= -60)
-          code = std::string("EPSG:327") + (zone < 10 ? "0" : "") + Utils::toString(zone);
+          code = std::string("EPSG:327");
        else
-          throw pdal_error("BPF file contains an invalid UTM zone");
+          throwError("BPF file contains an invalid UTM zone " +
+            Utils::toString(zone));
+       code += (zone < 10 ? "0" : "") + Utils::toString(zone);
+    }
+    else if (m_header.m_coordType == static_cast<int>(BpfCoordType::TCR))
+    {
+        // TCR is ECEF meters, or EPSG:4978
+        // According to the 1.0 spec, the m_coordId must be 1 to be
+        // valid.
+        if (m_header.m_coordId == 1)
+            code = std::string("EPSG:4978");
+        else
+        {
+            std::ostringstream oss;
+            oss << "BPF has ECEF/TCR coordinate type defined, but the ID of '"
+                <<  m_header.m_coordId << "' is invalid";
+            throwError(oss.str());
+        }
     }
     else
     {
-       //BPF supports something called Terrestrial Centered Rotational (BpfCoordType::TCR) and East North Up (BpfCoordType::ENU)
-       //which we can figure out when we run into a file with these coordinate systems.
-       throw pdal_error("BPF file contains unsupported coordinate system");
+       // BPF also supports something East North Up (BpfCoordType::ENU)
+       // which we can figure out when we run into a file with these
+       // coordinate systems.
+        std::ostringstream oss;
+        oss << "BPF file contains unsupported coordinate system with "
+            << "coordinate type: '" << m_header.m_coordType
+            << "' and coordinate id: '" << m_header.m_coordId << "'";
+       throwError(oss.str());
     }
     SpatialReference srs(code);
     setSpatialReference(srs);
@@ -160,12 +197,7 @@ void BpfReader::initialize()
     // Fast forward file to end of header as reported by base header.
     std::streampos pos = m_stream.position();
     if (pos > m_header.m_len)
-    {
-        std::ostringstream oss;
-        oss << getName() << ": BPF Header length exceeded that reported by "
-            "file.";
-        throw pdal_error(oss.str());
-    }
+        throwError("BPF Header length exceeded that reported by file.");
     m_stream.close();
 }
 
@@ -251,6 +283,7 @@ void BpfReader::ready(PointTableRef)
     m_stream.seek(m_header.m_len);
     m_index = 0;
     m_start = m_stream.position();
+#ifdef PDAL_HAVE_ZLIB
     if (m_header.m_compression)
     {
         m_deflateBuf.resize(numPoints() * m_dims.size() * sizeof(float));
@@ -264,6 +297,7 @@ void BpfReader::ready(PointTableRef)
         m_charbuf.initialize(m_deflateBuf.data(), m_deflateBuf.size(), m_start);
         m_stream.pushStream(new std::istream(&m_charbuf));
     }
+#endif // PDAL_HAVE_ZLIB
 }
 
 
@@ -277,6 +311,9 @@ void BpfReader::done(PointTableRef)
 
 bool BpfReader::processOne(PointRef& point)
 {
+    if (eof() || m_index >= m_count)
+        return false;
+
     switch (m_header.m_pointFormat)
     {
     case BpfFormat::PointMajor:
@@ -289,7 +326,7 @@ bool BpfReader::processOne(PointRef& point)
         readByteMajor(point);
         break;
     }
-    return !eof() && (m_index < m_count);
+    return true;
 }
 
 
@@ -308,24 +345,6 @@ point_count_t BpfReader::read(PointViewPtr data, point_count_t count)
 }
 
 
-size_t BpfReader::readBlock(std::vector<char>& outBuf, size_t index)
-{
-    uint32_t finalBytes;
-    uint32_t compressBytes;
-
-    m_stream >> finalBytes;
-    m_stream >> compressBytes;
-
-    std::vector<char> in(compressBytes);
-
-    // Fill the input bytes from the stream.
-    m_stream.get(in);
-    int ret = inflate(in.data(), compressBytes,
-        outBuf.data() + index, finalBytes);
-    return (ret ? 0 : finalBytes);
-}
-
-
 bool BpfReader::eof()
 {
     return m_index >= numPoints();
@@ -408,6 +427,7 @@ void BpfReader::readDimMajor(PointRef& point)
             m_streams.emplace_back(new ILeStream());
             m_streams.back()->open(m_filename);
 
+#ifdef PDAL_HAVE_ZLIB
             if (m_header.m_compression)
             {
                 m_charbufs.emplace_back(new Charbuf());
@@ -417,6 +437,7 @@ void BpfReader::readDimMajor(PointRef& point)
                 m_streams.back()->pushStream(
                         new std::istream(m_charbufs.back().get()));
             }
+#endif // PDAL_HAVE_ZLIB
 
             m_streams.back()->seek(m_start + offset);
         }
@@ -617,6 +638,25 @@ void BpfReader::seekByteMajor(size_t dimIdx, size_t byteIdx, PointId ptIdx)
 }
 
 
+#ifdef PDAL_HAVE_ZLIB
+size_t BpfReader::readBlock(std::vector<char>& outBuf, size_t index)
+{
+    uint32_t finalBytes;
+    uint32_t compressBytes;
+
+    m_stream >> finalBytes;
+    m_stream >> compressBytes;
+
+    std::vector<char> in(compressBytes);
+
+    // Fill the input bytes from the stream.
+    m_stream.get(in);
+    int ret = inflate(in.data(), compressBytes,
+        outBuf.data() + index, finalBytes);
+    return (ret ? 0 : finalBytes);
+}
+
+
 int BpfReader::inflate(char *buf, uint32_t insize,
     char *outbuf, uint32_t outsize)
 {
@@ -644,5 +684,6 @@ int BpfReader::inflate(char *buf, uint32_t insize,
     (void)inflateEnd(&strm);
     return ret == Z_STREAM_END ? 0 : -1;
 }
+#endif // PDAL_HAVE_ZLIB
 
 } //namespace pdal
