@@ -37,18 +37,19 @@
 
 #include <pdal/GDALUtils.hpp>
 #include <pdal/PointView.hpp>
-#include <pdal/pdal_macros.hpp>
 
 namespace pdal
 {
 
-static PluginInfo const s_info = PluginInfo(
-        "writers.gdal",
-        "Write a point cloud as a GDAL raster.",
-        "http://pdal.io/stages/writers.gdal.html");
+static StaticPluginInfo const s_info
+{
+    "writers.gdal",
+    "Write a point cloud as a GDAL raster.",
+    "http://pdal.io/stages/writers.gdal.html",
+    { "tif", "tiff", "vrt" }
+};
 
-
-CREATE_STATIC_PLUGIN(1, 0, GDALWriter, Writer, s_info);
+CREATE_STATIC_STAGE(GDALWriter, s_info)
 
 std::string GDALWriter::getName() const
 {
@@ -68,9 +69,13 @@ void GDALWriter::addArgs(ProgramArgs& args)
         m_options);
     args.add("output_type", "Statistics produced ('min', 'max', 'mean', "
         "'idw', 'count', 'stdev' or 'all')", m_outputTypeString, {"all"} );
+    args.add("data_type", "Data type for output grid (\"int8\", \"uint64\", "
+        "\"float\", etc.)", m_dataType, Dimension::Type::Double);
     args.add("window_size", "Cell distance for fallback interpolation",
         m_windowSize);
-    args.add("nodata", "No data value", m_noData, -9999.0);
+    // Nan is a sentinal value to say that no value was set for nodata.
+    args.add("nodata", "No data value", m_noData,
+        std::numeric_limits<double>::quiet_NaN());
     args.add("dimension", "Dimension to use", m_interpDimString, "Z");
     args.add("bounds", "Bounds of data.  Required in streaming mode.",
         m_bounds);
@@ -140,8 +145,15 @@ void GDALWriter::createGrid(BOX2D bounds)
     m_curBounds = bounds;
     size_t width = ((m_curBounds.maxx - m_curBounds.minx) / m_edgeLength) + 1;
     size_t height = ((m_curBounds.maxy - m_curBounds.miny) / m_edgeLength) + 1;
-    m_grid.reset(new GDALGrid(width, height, m_edgeLength, m_radius, m_noData,
-        m_outputTypes, m_windowSize));
+    try
+    {
+        m_grid.reset(new GDALGrid(width, height, m_edgeLength, m_radius,
+                    m_outputTypes, m_windowSize));
+    }
+    catch (GDALGrid::error& err)
+    {
+        throwError(err.what());
+    }
 }
 
 
@@ -207,6 +219,10 @@ bool GDALWriter::processOne(PointRef& point)
 
 void GDALWriter::doneFile()
 {
+    if (!m_grid) {
+        throw pdal_error("Unable to write GDAL data, grid is uninitialized. You "
+                "might have provided the GDALWriter zero points.");
+    }
     std::array<double, 6> pixelToPos;
 
     pixelToPos[0] = m_curBounds.minx;
@@ -220,30 +236,35 @@ void GDALWriter::doneFile()
     m_grid->finalize();
 
     gdal::GDALError err = raster.open(m_grid->width(), m_grid->height(),
-        m_grid->numBands(), Dimension::Type::Double, m_grid->noData(),
-        m_options);
+        m_grid->numBands(), m_dataType, m_noData, m_options);
+
     if (err != gdal::GDALError::None)
         throwError(raster.errorMsg());
     int bandNum = 1;
-    uint8_t *buf;
-    buf = m_grid->data("min");
-    if (buf)
-        raster.writeBand(buf, bandNum++, "min");
-    buf = m_grid->data("max");
-    if (buf)
-        raster.writeBand(buf, bandNum++, "max");
-    buf = m_grid->data("mean");
-    if (buf)
-        raster.writeBand(buf, bandNum++, "mean");
-    buf = m_grid->data("idw");
-    if (buf)
-        raster.writeBand(buf, bandNum++, "idw");
-    buf = m_grid->data("count");
-    if (buf)
-        raster.writeBand(buf, bandNum++, "count");
-    buf = m_grid->data("stdev");
-    if (buf)
-        raster.writeBand(buf, bandNum++, "stdev");
+
+    // Perhaps the grid should return an iterator, which would work as well.
+    double *src;
+    src = m_grid->data("min");
+    double srcNoData = std::numeric_limits<double>::quiet_NaN();
+    if (src && err == gdal::GDALError::None)
+        err = raster.writeBand(src, srcNoData, bandNum++, "min");
+    src = m_grid->data("max");
+    if (src && err == gdal::GDALError::None)
+        err = raster.writeBand(src, srcNoData, bandNum++, "max");
+    src = m_grid->data("mean");
+    if (src && err == gdal::GDALError::None)
+        err = raster.writeBand(src, srcNoData, bandNum++, "mean");
+    src = m_grid->data("idw");
+    if (src && err == gdal::GDALError::None)
+        err = raster.writeBand(src, srcNoData, bandNum++, "idw");
+    src = m_grid->data("count");
+    if (src && err == gdal::GDALError::None)
+        err = raster.writeBand(src, srcNoData, bandNum++, "count");
+    src = m_grid->data("stdev");
+    if (src && err == gdal::GDALError::None)
+        err = raster.writeBand(src, srcNoData, bandNum++, "stdev");
+    if (err != gdal::GDALError::None)
+        throwError(raster.errorMsg());
 
     getMetadata().addList("filename", m_filename);
 }
