@@ -33,8 +33,8 @@
 ****************************************************************************/
 
 #include "Ilvis2Reader.hpp"
+#include "Ilvis2MetadataReader.hpp"
 #include <pdal/util/FileUtils.hpp>
-#include <pdal/pdal_macros.hpp>
 #include <pdal/util/ProgramArgs.hpp>
 
 #include <algorithm>
@@ -43,12 +43,14 @@
 namespace pdal
 {
 
-static PluginInfo const s_info = PluginInfo(
+static StaticPluginInfo const s_info
+{
     "readers.ilvis2",
     "ILVIS2 Reader",
-    "http://pdal.io/stages/readers.ilvis2.html" );
+    "http://pdal.io/stages/readers.ilvis2.html"
+};
 
-CREATE_STATIC_PLUGIN(1, 0, Ilvis2Reader, Reader, s_info)
+CREATE_STATIC_STAGE(Ilvis2Reader, s_info)
 
 std::string Ilvis2Reader::getName() const { return s_info.name; }
 
@@ -115,19 +117,6 @@ void Ilvis2Reader::addDimensions(PointLayoutPtr layout)
 }
 
 
-Dimension::IdList Ilvis2Reader::getDefaultDimensions()
-{
-    using namespace pdal::Dimension;
-    Dimension::IdList ids;
-
-    ids.push_back(Id::GpsTime);
-    ids.push_back(Id::Y);
-    ids.push_back(Id::X);
-    ids.push_back(Id::Z);
-    return ids;
-}
-
-
 void Ilvis2Reader::initialize(PointTableRef)
 {
     if (!m_metadataFile.empty() && !FileUtils::fileExists(m_metadataFile))
