@@ -32,23 +32,24 @@
 * OF SUCH DAMAGE.
 ****************************************************************************/
 
+#include <pdal/compression/LazPerfVlrCompression.hpp>
+
 #include "LasReader.hpp"
 
 #include <sstream>
 #include <string.h>
 
+#include <pdal/pdal_features.hpp>
 #include <pdal/Metadata.hpp>
 #include <pdal/PointView.hpp>
 #include <pdal/QuickInfo.hpp>
 #include <pdal/util/Extractor.hpp>
 #include <pdal/util/IStream.hpp>
-#include <pdal/pdal_macros.hpp>
 #include <pdal/util/ProgramArgs.hpp>
 
 #include "GeotiffSupport.hpp"
 #include "LasHeader.hpp"
 #include "LasVLR.hpp"
-#include "LasZipPoint.hpp"
 
 namespace pdal
 {
@@ -64,23 +65,40 @@ struct invalid_stream : public std::runtime_error
 
 } // unnamed namespace
 
+LasReader::LasReader() : m_decompressor(nullptr), m_index(0)
+{}
+
+
+LasReader::~LasReader()
+{
+#ifdef PDAL_HAVE_LAZPERF
+    delete m_decompressor;
+#endif
+}
+
+
 void LasReader::addArgs(ProgramArgs& args)
 {
     addSpatialReferenceArg(args);
     args.add("extra_dims", "Dimensions to assign to extra byte data",
         m_extraDimSpec);
     args.add("compression", "Decompressor to use", m_compression, "EITHER");
+    args.add("use_eb_vlr", "Use extra bytes VLR for 1.0 - 1.3 files",
+        m_useEbVlr);
+    args.add("ignore_vlr", "VLR userid/recordid to ignore", m_ignoreVLROption);
 }
 
 
-static PluginInfo const s_info = PluginInfo(
+static StaticPluginInfo const s_info {
     "readers.las",
     "ASPRS LAS 1.0 - 1.4 read support. LASzip support is also \n" \
         "enabled through this driver if LASzip was found during \n" \
         "compilation.",
-    "http://pdal.io/stages/readers.las.html" );
+    "http://pdal.io/stages/readers.las.html",
+    { "las", "laz" }
+};
 
-CREATE_STATIC_PLUGIN(1, 0, LasReader, Reader, s_info)
+CREATE_STATIC_STAGE(LasReader, s_info)
 
 std::string LasReader::getName() const { return s_info.name; }
 
@@ -108,17 +126,8 @@ QuickInfo LasReader::inspect()
 }
 
 
-void LasReader::initializeLocal(PointTableRef table, MetadataNode& m)
+void LasReader::handleCompressionOption()
 {
-    try
-    {
-        m_extraDims = LasUtils::parse(m_extraDimSpec);
-    }
-    catch (const LasUtils::error& err)
-    {
-        throwError(err.what());
-    }
-
     std::string compression = Utils::toupper(m_compression);
 #if defined(PDAL_HAVE_LAZPERF) && defined(PDAL_HAVE_LASZIP)
     if (compression == "EITHER")
@@ -147,18 +156,39 @@ void LasReader::initializeLocal(PointTableRef table, MetadataNode& m)
 
     // Set case-corrected value.
     m_compression = compression;
-    m_error.setFilename(m_filename);
+}
+
+
+void LasReader::initializeLocal(PointTableRef table, MetadataNode& m)
+{
+    try
+    {
+        m_extraDims = LasUtils::parse(m_extraDimSpec, false);
+    }
+    catch (const LasUtils::error& err)
+    {
+        throwError(err.what());
+    }
+
+    try
+    {
+        m_ignoreVLRs = LasUtils::parseIgnoreVLRs(m_ignoreVLROption);
+    }
+    catch (const LasUtils::error& err)
+    {
+        throwError(err.what());
+    }
 
-    m_error.setLog(log());
     m_header.setLog(log());
-    createStream();
 
+    createStream();
     std::istream *stream(m_streamIf->m_istream);
 
     stream->seekg(0);
     ILeStream in(stream);
     try
     {
+        // This also reads the extended VLRs at the end of the data.
         in >> m_header;
     }
     catch (const LasHeader::error& e)
@@ -166,11 +196,25 @@ void LasReader::initializeLocal(PointTableRef table, MetadataNode& m)
         throwError(e.what());
     }
 
+    for (auto i: m_ignoreVLRs)
+    {
+        if (i.m_recordId)
+            m_header.removeVLR(i.m_userId, i.m_recordId);
+        else
+            m_header.removeVLR(i.m_userId);
+    }
+
+    if (m_header.compressed())
+        handleCompressionOption();
+#ifdef PDAL_HAVE_LASZIP
+    m_laszip = nullptr;
+#endif
+
     if (!m_header.pointFormatSupported())
         throwError("Unsupported LAS input point format: " +
             Utils::toString((int)m_header.pointFormat()) + ".");
 
-    if (m_header.versionAtLeast(1, 4))
+    if (m_header.versionAtLeast(1, 4) || m_useEbVlr)
         readExtraBytesVlr();
     setSrs(m);
     MetadataNode forward = table.privateMetadata("lasforward");
@@ -181,6 +225,19 @@ void LasReader::initializeLocal(PointTableRef table, MetadataNode& m)
 }
 
 
+void LasReader::handleLaszip(int result)
+{
+#ifdef PDAL_HAVE_LASZIP
+    if (result)
+    {
+        char *buf;
+        laszip_get_error(m_laszip, &buf);
+        throwError(buf);
+    }
+#endif
+}
+
+
 void LasReader::ready(PointTableRef table)
 {
     createStream();
@@ -192,45 +249,23 @@ void LasReader::ready(PointTableRef table)
 #ifdef PDAL_HAVE_LASZIP
         if (m_compression == "LASZIP")
         {
-            LasVLR *vlr = m_header.findVlr(LASZIP_USER_ID,
-                LASZIP_RECORD_ID);
-            try
-            {
-                m_zipPoint.reset(new LasZipPoint(vlr));
-            }
-            catch (const LasZipPoint::error& err)
-            {
-                throwError(err.what());
-            }
+            laszip_BOOL compressed;
 
-            if (!m_unzipper)
-            {
-                m_unzipper.reset(new LASunzipper());
-
-                stream->seekg(m_header.pointOffset(), std::ios::beg);
-
-                // Once we open the zipper, don't touch the stream until the
-                // zipper is closed or bad things happen.
-                if (!m_unzipper->open(*stream, m_zipPoint->GetZipper()))
-                {
-                    std::ostringstream oss;
-                    const char* err = m_unzipper->get_error();
-                    if (err == NULL)
-                        err = "(unknown error)";
-                    throwError("Failed to open LASzip stream: " +
-                        std::string(err) + ".");
-                }
-            }
+            handleLaszip(laszip_create(&m_laszip));
+            handleLaszip(laszip_open_reader_stream(m_laszip, *stream,
+                &compressed));
+            handleLaszip(laszip_get_point_pointer(m_laszip, &m_laszipPoint));
         }
 #endif
 
 #ifdef PDAL_HAVE_LAZPERF
         if (m_compression == "LAZPERF")
         {
-            LasVLR *vlr = m_header.findVlr(LASZIP_USER_ID,
+            const LasVLR *vlr = m_header.findVlr(LASZIP_USER_ID,
                 LASZIP_RECORD_ID);
-            m_decompressor.reset(new LazPerfVlrDecompressor(*stream,
-                vlr->data(), m_header.pointOffset()));
+            delete m_decompressor;
+            m_decompressor = new LazPerfVlrDecompressor(*stream,
+                vlr->data(), m_header.pointOffset());
             m_decompressorBuf.resize(m_decompressor->pointSize());
         }
 #endif
@@ -245,11 +280,36 @@ void LasReader::ready(PointTableRef table)
 }
 
 
+namespace
+{
+
+void addForwardMetadata(MetadataNode& forward, MetadataNode& m,
+    const std::string& name, double val, const std::string description,
+    size_t precision)
+{
+    MetadataNode n = m.add(name, val, description, precision);
+
+    // If the entry doesn't already exist, just add it.
+    MetadataNode f = forward.findChild(name);
+    if (!f.valid())
+    {
+        forward.add(n);
+        return;
+    }
+
+    // If the old value and new values aren't the same, set an invalid flag.
+    MetadataNode temp = f.addOrUpdate("temp", val);
+    if (f.value<std::string>() != temp.value<std::string>())
+        forward.addOrUpdate(name + "INVALID", "");
+}
+
+}
+
 // Store data in the normal metadata place.  Also store it in the private
 // lasforward metadata node.
 template <typename T>
 void addForwardMetadata(MetadataNode& forward, MetadataNode& m,
-    const std::string& name, T val, const std::string description = "")
+    const std::string& name, T val, const std::string description)
 {
     MetadataNode n = m.add(name, val, description);
 
@@ -301,7 +361,8 @@ void LasReader::extractHeaderMetadata(MetadataNode& forward, MetadataNode& m)
 
     addForwardMetadata(forward, m, "project_id", m_header.projectId(),
         "Project ID.");
-    addForwardMetadata(forward, m, "system_id", m_header.systemId());
+    addForwardMetadata(forward, m, "system_id", m_header.systemId(),
+        "Generating system ID.");
     addForwardMetadata(forward, m, "software_id", m_header.softwareId(),
         "Generating software description.");
     addForwardMetadata(forward, m, "creation_doy", m_header.creationDOY(),
@@ -312,17 +373,17 @@ void LasReader::extractHeaderMetadata(MetadataNode& forward, MetadataNode& m)
         "The year, expressed as a four digit number, in which the file was "
         "created.");
     addForwardMetadata(forward, m, "scale_x", m_header.scaleX(),
-        "The scale factor for X values.");
+        "The scale factor for X values.", 20);
     addForwardMetadata(forward, m, "scale_y", m_header.scaleY(),
-        "The scale factor for Y values.");
+        "The scale factor for Y values.", 20);
     addForwardMetadata(forward, m, "scale_z", m_header.scaleZ(),
-        "The scale factor for Z values.");
+        "The scale factor for Z values.", 20);
     addForwardMetadata(forward, m, "offset_x", m_header.offsetX(),
-        "The offset for X values.");
+        "The offset for X values.", 20);
     addForwardMetadata(forward, m, "offset_y", m_header.offsetY(),
-        "The offset for Y values.");
+        "The offset for Y values.", 20);
     addForwardMetadata(forward, m, "offset_z", m_header.offsetZ(),
-        "The offset for Z values.");
+        "The offset for Z values.", 20);
 
     m.add("header_size", m_header.vlrOffset(),
         "The size, in bytes, of the header block, including any extension "
@@ -361,12 +422,13 @@ void LasReader::extractHeaderMetadata(MetadataNode& forward, MetadataNode& m)
         "number of point records within the file.");
 
     // PDAL metadata VLR
-    LasVLR *vlr = m_header.findVlr("PDAL", 12);
+    const LasVLR *vlr = m_header.findVlr("PDAL", 12);
     if (vlr)
     {
         const char *pos = vlr->data();
         size_t size = vlr->dataLen();
-        m.addWithType("pdal_metadata", std::string(pos, size), "json", "PDAL Processing Metadata");
+        m.addWithType("pdal_metadata", std::string(pos, size), "json",
+            "PDAL Processing Metadata");
     }
     //
     // PDAL pipeline VLR
@@ -375,15 +437,15 @@ void LasReader::extractHeaderMetadata(MetadataNode& forward, MetadataNode& m)
     {
         const char *pos = vlr->data();
         size_t size = vlr->dataLen();
-        m.addWithType("pdal_pipeline", std::string(pos, size), "json", "PDAL Processing Pipeline");
+        m.addWithType("pdal_pipeline", std::string(pos, size), "json",
+            "PDAL Processing Pipeline");
     }
-
 }
 
 
 void LasReader::readExtraBytesVlr()
 {
-    LasVLR *vlr = m_header.findVlr(SPEC_USER_ID,
+    const LasVLR *vlr = m_header.findVlr(SPEC_USER_ID,
         EXTRA_BYTES_RECORD_ID);
     if (!vlr)
         return;
@@ -444,9 +506,11 @@ void LasReader::extractVlrMetadata(MetadataNode& forward, MetadataNode& m)
 
         std::ostringstream name;
         name << "vlr_" << i++;
-        MetadataNode vlrNode = m.addEncoded(name.str(),
-            (const uint8_t *)vlr.data(), vlr.dataLen(), vlr.description());
+        MetadataNode vlrNode(name.str());
+        m.add(vlrNode);
 
+        vlrNode.addEncoded("data",
+            (const uint8_t *)vlr.data(), vlr.dataLen(), vlr.description());
         vlrNode.add("user_id", vlr.userId(),
             "User ID of the record or pre-defined value from the "
             "specification.");
@@ -523,17 +587,8 @@ bool LasReader::processOne(PointRef& point)
 #ifdef PDAL_HAVE_LASZIP
         if (m_compression == "LASZIP")
         {
-            if (!m_unzipper->read(m_zipPoint->m_lz_point))
-            {
-                std::string error = "Error reading compressed point data: ";
-                const char* err = m_unzipper->get_error();
-                if (!err)
-                    err = "(unknown error)";
-                error += std::string(err) + ".";
-                throwError(error);
-            }
-            loadPoint(point, (char *)m_zipPoint->m_lz_point_data.data(),
-                pointLen);
+            handleLaszip(laszip_read_point(m_laszip));
+            loadPoint(point, *m_laszipPoint);
         }
 #endif
 
@@ -647,6 +702,17 @@ point_count_t LasReader::readFileBlock(std::vector<char>& buf,
 }
 
 
+#ifdef PDAL_HAVE_LASZIP
+void LasReader::loadPoint(PointRef& point, laszip_point& p)
+{
+    if (m_header.has14Format())
+        loadPointV14(point, p);
+    else
+        loadPointV10(point, p);
+}
+#endif // PDAL_HAVE_LASZIP
+
+
 void LasReader::loadPoint(PointRef& point, char *buf, size_t bufsize)
 {
     if (m_header.has14Format())
@@ -656,6 +722,48 @@ void LasReader::loadPoint(PointRef& point, char *buf, size_t bufsize)
 }
 
 
+#ifdef PDAL_HAVE_LASZIP
+void LasReader::loadPointV10(PointRef& point, laszip_point& p)
+{
+    const LasHeader& h = m_header;
+
+    double x = p.X * h.scaleX() + h.offsetX();
+    double y = p.Y * h.scaleY() + h.offsetY();
+    double z = p.Z * h.scaleZ() + h.offsetZ();
+
+    point.setField(Dimension::Id::X, x);
+    point.setField(Dimension::Id::Y, y);
+    point.setField(Dimension::Id::Z, z);
+    point.setField(Dimension::Id::Intensity, p.intensity);
+    point.setField(Dimension::Id::ReturnNumber, p.return_number);
+    point.setField(Dimension::Id::NumberOfReturns, p.number_of_returns);
+    point.setField(Dimension::Id::ScanDirectionFlag, p.scan_direction_flag);
+    point.setField(Dimension::Id::EdgeOfFlightLine, p.edge_of_flight_line);
+    uint8_t classification = p.classification | (p.synthetic_flag << 5) |
+        (p.keypoint_flag << 6) | (p.withheld_flag << 7);
+    point.setField(Dimension::Id::Classification, classification);
+    point.setField(Dimension::Id::ScanAngleRank, p.scan_angle_rank);
+    point.setField(Dimension::Id::UserData, p.user_data);
+    point.setField(Dimension::Id::PointSourceId, p.point_source_ID);
+
+    if (h.hasTime())
+        point.setField(Dimension::Id::GpsTime, p.gps_time);
+
+    if (h.hasColor())
+    {
+        point.setField(Dimension::Id::Red, p.rgb[0]);
+        point.setField(Dimension::Id::Green, p.rgb[1]);
+        point.setField(Dimension::Id::Blue, p.rgb[2]);
+    }
+
+    if (m_extraDims.size())
+    {
+        LeExtractor extractor((const char *)p.extra_bytes, p.num_extra_bytes);
+        loadExtraDims(extractor, point);
+    }
+}
+#endif // PDAL_HAVE_LASZIP
+
 void LasReader::loadPointV10(PointRef& point, char *buf, size_t bufsize)
 {
     LeExtractor istream(buf, bufsize);
@@ -684,12 +792,6 @@ void LasReader::loadPointV10(PointRef& point, char *buf, size_t bufsize)
     uint8_t scanDirFlag = (flags >> 6) & 0x01;
     uint8_t flight = (flags >> 7) & 0x01;
 
-    if (returnNum == 0 || returnNum > 5)
-        m_error.returnNumWarning(returnNum);
-
-    if (numReturns == 0 || numReturns > 5)
-        m_error.numReturnsWarning(numReturns);
-
     point.setField(Dimension::Id::X, x);
     point.setField(Dimension::Id::Y, y);
     point.setField(Dimension::Id::Z, z);
@@ -721,9 +823,56 @@ void LasReader::loadPointV10(PointRef& point, char *buf, size_t bufsize)
 
     if (m_extraDims.size())
         loadExtraDims(istream, point);
-
 }
 
+
+#ifdef PDAL_HAVE_LASZIP
+void LasReader::loadPointV14(PointRef& point, laszip_point& p)
+{
+    const LasHeader& h = m_header;
+
+    double x = p.X * h.scaleX() + h.offsetX();
+    double y = p.Y * h.scaleY() + h.offsetY();
+    double z = p.Z * h.scaleZ() + h.offsetZ();
+
+    point.setField(Dimension::Id::X, x);
+    point.setField(Dimension::Id::Y, y);
+    point.setField(Dimension::Id::Z, z);
+    point.setField(Dimension::Id::Intensity, p.intensity);
+    point.setField(Dimension::Id::ReturnNumber, p.extended_return_number);
+    point.setField(Dimension::Id::NumberOfReturns,
+        p.extended_number_of_returns);
+    point.setField(Dimension::Id::ClassFlags, p.extended_classification_flags);
+    point.setField(Dimension::Id::ScanChannel, p.extended_scanner_channel);
+    point.setField(Dimension::Id::ScanDirectionFlag, p.scan_direction_flag);
+    point.setField(Dimension::Id::EdgeOfFlightLine, p.edge_of_flight_line);
+    point.setField(Dimension::Id::Classification, p.extended_classification);
+    point.setField(Dimension::Id::ScanAngleRank, p.extended_scan_angle * .006);
+    point.setField(Dimension::Id::UserData, p.user_data);
+    point.setField(Dimension::Id::PointSourceId, p.point_source_ID);
+    point.setField(Dimension::Id::GpsTime, p.gps_time);
+
+    if (h.hasColor())
+    {
+        point.setField(Dimension::Id::Red, p.rgb[0]);
+        point.setField(Dimension::Id::Green, p.rgb[1]);
+        point.setField(Dimension::Id::Blue, p.rgb[2]);
+    }
+
+    if (h.hasInfrared())
+    {
+        point.setField(Dimension::Id::Infrared, p.rgb[3]);
+    }
+
+    if (m_extraDims.size())
+    {
+        LeExtractor extractor((const char *)p.extra_bytes, p.num_extra_bytes);
+        loadExtraDims(extractor, point);
+    }
+}
+#endif  // PDAL_HAVE_LASZIP
+
+
 void LasReader::loadPointV14(PointRef& point, char *buf, size_t bufsize)
 {
     LeExtractor istream(buf, bufsize);
@@ -822,8 +971,11 @@ void LasReader::loadExtraDims(LeExtractor& istream, PointRef& point)
 void LasReader::done(PointTableRef)
 {
 #ifdef PDAL_HAVE_LASZIP
-    m_zipPoint.reset();
-    m_unzipper.reset();
+    if (m_laszip)
+    {
+        handleLaszip(laszip_close_reader(m_laszip));
+        handleLaszip(laszip_destroy(m_laszip));
+    }
 #endif
     m_streamIf.reset();
 }
