@@ -42,17 +42,19 @@
 #include <pdal/PDALUtils.hpp>
 #include <pdal/SpatialReference.hpp>
 #include <pdal/StageFactory.hpp>
-#include <pdal/pdal_macros.hpp>
 
 namespace pdal
 {
 
-static PluginInfo const s_info = PluginInfo(
+static StaticPluginInfo const s_info
+{
     "readers.optech",
     "Optech reader support.",
-    "http://pdal.io/stages/readers.optech.html" );
+    "http://pdal.io/stages/readers.optech.html",
+    { "csd" }
+};
 
-CREATE_STATIC_PLUGIN(1, 0, OptechReader, Reader, s_info);
+CREATE_STATIC_STAGE(OptechReader, s_info)
 
 std::string OptechReader::getName() const
 {
@@ -82,22 +84,6 @@ OptechReader::OptechReader()
 }
 
 
-Dimension::IdList OptechReader::getDefaultDimensions()
-{
-    Dimension::IdList dims;
-    dims.push_back(Dimension::Id::X);
-    dims.push_back(Dimension::Id::Y);
-    dims.push_back(Dimension::Id::Z);
-    dims.push_back(Dimension::Id::GpsTime);
-    dims.push_back(Dimension::Id::ReturnNumber);
-    dims.push_back(Dimension::Id::NumberOfReturns);
-    dims.push_back(Dimension::Id::EchoRange);
-    dims.push_back(Dimension::Id::Intensity);
-    dims.push_back(Dimension::Id::ScanAngleRank);
-    return dims;
-}
-
-
 const CsdHeader& OptechReader::getHeader() const { return m_header; }
 
 
@@ -136,10 +122,11 @@ void OptechReader::initialize()
 
 void OptechReader::addDimensions(PointLayoutPtr layout)
 {
-    for (auto it : getDefaultDimensions())
-    {
-        layout->registerDim(it);
-    }
+    using namespace Dimension;
+
+    layout->registerDims( { Id::X, Id::Y, Id::Z, Id::GpsTime, Id::ReturnNumber,
+        Id::NumberOfReturns, Id::EchoRange, Id::Intensity,
+        Id::ScanAngleRank } );
 }
 
 
