@@ -167,7 +167,6 @@ Word #       Content
 #include "QfitReader.hpp"
 
 #include <pdal/PointView.hpp>
-#include <pdal/pdal_macros.hpp>
 #include <pdal/util/Extractor.hpp>
 #include <pdal/util/portable_endian.hpp>
 #include <pdal/util/ProgramArgs.hpp>
@@ -175,20 +174,20 @@ Word #       Content
 #include <algorithm>
 #include <map>
 
-#ifdef PDAL_COMPILER_MSVC
-#  pragma warning(disable: 4127)  // conditional expression is constant
-#endif
-
+#pragma warning(disable: 4127)  // conditional expression is constant
 
 namespace pdal
 {
 
-static PluginInfo const s_info = PluginInfo(
+static StaticPluginInfo const s_info
+{
     "readers.qfit",
     "QFIT Reader",
-    "http://pdal.io/stages/readers.qfit.html" );
+    "http://pdal.io/stages/readers.qfit.html",
+    { "qi" }
+};
 
-CREATE_STATIC_PLUGIN(1, 0, QfitReader, Reader, s_info)
+CREATE_STATIC_STAGE(QfitReader, s_info)
 
 std::string QfitReader::getName() const { return s_info.name; }
 
@@ -401,30 +400,6 @@ point_count_t QfitReader::read(PointViewPtr data, point_count_t count)
 }
 
 
-Dimension::IdList QfitReader::getDefaultDimensions()
-{
-    Dimension::IdList ids;
-
-    ids.push_back(Dimension::Id::OffsetTime);
-    ids.push_back(Dimension::Id::Y);
-    ids.push_back(Dimension::Id::X);
-    ids.push_back(Dimension::Id::Z);
-    ids.push_back(Dimension::Id::StartPulse);
-    ids.push_back(Dimension::Id::ReflectedPulse);
-    ids.push_back(Dimension::Id::Azimuth);
-    ids.push_back(Dimension::Id::Pitch);
-    ids.push_back(Dimension::Id::Roll);
-    ids.push_back(Dimension::Id::Pdop);
-    ids.push_back(Dimension::Id::PulseWidth);
-    ids.push_back(Dimension::Id::PassiveSignal);
-    ids.push_back(Dimension::Id::PassiveY);
-    ids.push_back(Dimension::Id::PassiveX);
-    ids.push_back(Dimension::Id::PassiveZ);
-
-    return ids;
-}
-
-
 void QfitReader::done(PointTableRef)
 {
     m_istream.reset();
