@@ -36,19 +36,21 @@
 
 #include <pdal/PointView.hpp>
 #include <pdal/util/Extractor.hpp>
-#include <pdal/pdal_macros.hpp>
 
 #include <map>
 
 namespace pdal
 {
 
-static PluginInfo const s_info = PluginInfo(
+static StaticPluginInfo const s_info
+{
     "readers.terrasolid",
     "TerraSolid Reader",
-    "http://pdal.io/stages/readers.terrasolid.html" );
+    "http://pdal.io/stages/readers.terrasolid.html",
+    { "bin" }
+};
 
-CREATE_STATIC_PLUGIN(1, 0, TerrasolidReader, Reader, s_info)
+CREATE_STATIC_STAGE(TerrasolidReader, s_info)
 
 std::string TerrasolidReader::getName() const { return s_info.name; }
 
@@ -129,31 +131,6 @@ void TerrasolidReader::addDimensions(PointLayoutPtr layout)
 }
 
 
-Dimension::IdList TerrasolidReader::getDefaultDimensions()
-{
-    using namespace Dimension;
-
-    IdList dims;
-
-    dims.push_back(Id::Classification);
-    dims.push_back(Id::PointSourceId);
-    dims.push_back(Id::ReturnNumber);
-    dims.push_back(Id::NumberOfReturns);
-    dims.push_back(Id::Flag);
-    dims.push_back(Id::Mark);
-    dims.push_back(Id::Intensity);
-    dims.push_back(Id::X);
-    dims.push_back(Id::Y);
-    dims.push_back(Id::Z);
-    dims.push_back(Id::Red);
-    dims.push_back(Id::Green);
-    dims.push_back(Id::Blue);
-    dims.push_back(Id::Alpha);
-    dims.push_back(Id::OffsetTime);
-    return dims;
-}
-
-
 void TerrasolidReader::ready(PointTableRef)
 {
     m_istream.reset(new IStream(m_filename));
