@@ -52,23 +52,30 @@ namespace pdal
 namespace eigen
 {
 
-Eigen::Vector3f computeCentroid(PointView& view, std::vector<PointId> ids)
+Eigen::Vector3d computeCentroid(PointView& view, std::vector<PointId> ids)
 {
     using namespace Eigen;
-
-    auto n = ids.size();
-
+    
     double mx, my, mz;
     mx = my = mz = 0.0;
+    point_count_t n(0);
     for (auto const& j : ids)
     {
-        mx += view.getFieldAs<double>(Dimension::Id::X, j);
-        my += view.getFieldAs<double>(Dimension::Id::Y, j);
-        mz += view.getFieldAs<double>(Dimension::Id::Z, j);
+        auto update = [&n](double value, double average)
+        {
+            double delta, delta_n;
+            delta = value - average;
+            delta_n = delta / n;
+            return average + delta_n;
+        };
+        n++;
+        mx = update(view.getFieldAs<double>(Dimension::Id::X, j), mx);
+        my = update(view.getFieldAs<double>(Dimension::Id::Y, j), my);
+        mz = update(view.getFieldAs<double>(Dimension::Id::Z, j), mz);
     }
 
-    Vector3f centroid;
-    centroid << mx/n, my/n, mz/n;
+    Vector3d centroid;
+    centroid << mx, my, mz;
 
     return centroid;
 }
@@ -79,7 +86,7 @@ Eigen::Matrix3f computeCovariance(PointView& view, std::vector<PointId> ids)
 
     auto n = ids.size();
 
-    Vector3f centroid = computeCentroid(view, ids);
+    Vector3d centroid = computeCentroid(view, ids);
 
     // demean the neighborhood
     MatrixXf A(3, n);
@@ -92,7 +99,7 @@ Eigen::Matrix3f computeCovariance(PointView& view, std::vector<PointId> ids)
         k++;
     }
 
-    return A * A.transpose();
+    return A * A.transpose() / (ids.size()-1);
 }
 
 uint8_t computeRank(PointView& view, std::vector<PointId> ids, double threshold)
@@ -279,7 +286,7 @@ Eigen::MatrixXd createMaxMatrix2(PointView& view, int rows, int cols,
             double y = bounds.miny + (r + 0.5) * cell_size;
 
             auto neighbors = kdi.radius(x, y, cell_size * std::sqrt(2.0));
-            
+
             double val(std::numeric_limits<double>::lowest());
             for (auto const& n : neighbors)
             {
@@ -472,7 +479,7 @@ std::vector<double> dilateDiamond(std::vector<double> data, size_t rows, size_t
 {
     std::vector<double> out(data.size(), std::numeric_limits<double>::lowest());
     std::vector<size_t> idx(5);
-    
+
     for (int iter = 0; iter < iterations; ++iter)
     {
         for (size_t col = 0; col < cols; ++col)
@@ -506,7 +513,7 @@ std::vector<double> erodeDiamond(std::vector<double> data, size_t rows, size_t c
 {
     std::vector<double> out(data.size(), std::numeric_limits<double>::max());
     std::vector<size_t> idx(5);
-    
+
     for (int iter = 0; iter < iterations; ++iter)
     {
         for (size_t col = 0; col < cols; ++col)
@@ -578,7 +585,7 @@ void writeMatrix(Eigen::MatrixXd data, const std::string& filename,
     Eigen::Matrix<float, Dynamic, Dynamic, RowMajor> dataRowMajor;
     dataRowMajor = data.cast<float>();
 
-    raster.writeBand((uint8_t *)dataRowMajor.data(), 1);
+    raster.writeBand((float*)dataRowMajor.data(), -9999.0f, 1);
 }
 
 Eigen::MatrixXd cleanDSM(Eigen::MatrixXd data)
