@@ -92,6 +92,8 @@ void PointLayout::registerDim(Dimension::Id id, Dimension::Type type)
 Dimension::Id PointLayout::assignDim(const std::string& name,
     Dimension::Type type)
 {
+    if (m_nextFree == Dimension::COUNT)
+        throw pdal_error("No dimension IDs remaining for assignment.");
     Dimension::Id id = (Dimension::Id)m_nextFree;
 
     auto di = m_propIds.find(name);
@@ -210,12 +212,6 @@ size_t PointLayout::pointSize() const
 }
 
 
-const Dimension::Detail* PointLayout::dimDetail(Dimension::Id id) const
-{
-    return &(m_detail[Utils::toNative(id)]);
-}
-
-
 // Update the point layout given dimension detail and the dimension's name.
 bool PointLayout::update(Dimension::Detail dd, const std::string& name)
 {
@@ -245,7 +241,7 @@ bool PointLayout::update(Dimension::Detail dd, const std::string& name)
     Dimension::Detail *cur = &(*di);
 
     {
-        auto sorter = [this](const Dimension::Detail& d1,
+        auto sorter = [](const Dimension::Detail& d1,
                 const Dimension::Detail& d2) -> bool
         {
             if (d1.size() > d2.size())
