@@ -36,19 +36,20 @@
 
 #include <lidar/MG4PointReader.h>
 #include <lidar/FileIO.h>
-#include <pdal/pdal_macros.hpp>
 #include <pdal/util/ProgramArgs.hpp>
 #include <pdal/DimUtil.hpp>
 
 namespace pdal
 {
 
-static PluginInfo const s_info = PluginInfo(
+static PluginInfo const s_info
+{
     "readers.mrsid",
     "MrSID Reader",
-    "http://www.pdal.io/stages/readers.mrsid.html" );
+    "http://pdal.io/stages/readers.mrsid.html"
+};
 
-CREATE_SHARED_PLUGIN(1, 0, MrsidReader, Reader, s_info)
+CREATE_SHARED_STAGE(MrsidReader, s_info)
 
 std::string MrsidReader::getName() const { return s_info.name; }
 
@@ -112,7 +113,7 @@ void MrsidReader::addDimensions(PointLayoutPtr layout)
     using namespace Dimension;
 
     if (!m_PS)
-        throw pdal_error("MrSID object not initialized!");
+        throwError("MrSID object not initialized.");
     const LizardTech::PointInfo& pointinfo = m_PS->getPointInfo();
 
     // add a map for PDAL names that aren't the same as LT ones (GPSTime vs Time)
