@@ -43,10 +43,7 @@
 
 #include <iostream>
 
-#ifdef PDAL_COMPILER_GCC
 #pragma GCC diagnostic ignored "-Wsign-compare"
-#endif
-
 
 using namespace pdal;
 
@@ -133,10 +130,12 @@ TEST(NitfReaderTest, optionSrs)
     EXPECT_TRUE(nitfReader);
     nitfReader->setOptions(nitfOpts);
 
+    std::string outfile(Support::temppath("temp.nitf"));
     Options lasOpts;
-    lasOpts.add("filename", "/dev/null");
+    lasOpts.add("filename", outfile);
 
     LasWriter writer;
+
     writer.setInput(*nitfReader);
     writer.setOptions(lasOpts);;
 
