@@ -32,10 +32,10 @@
 * OF SUCH DAMAGE.
 ****************************************************************************/
 
-#include <pdal/Compression.hpp>
+#include <pdal/pdal_features.hpp>
 #include <pdal/GDALUtils.hpp>
-#include <pdal/pdal_macros.hpp>
 #include <pdal/PDALUtils.hpp>
+#include <pdal/compression/LazPerfCompression.hpp>
 #include <pdal/util/FileUtils.hpp>
 #include <pdal/util/ProgramArgs.hpp>
 
@@ -44,12 +44,14 @@
 namespace pdal
 {
 
-static PluginInfo const s_info = PluginInfo(
+static PluginInfo const s_info
+{
     "readers.oci",
     "Read point cloud data from Oracle SDO_POINTCLOUD.",
-    "http://pdal.io/stages/readers.oci.html" );
+    "http://pdal.io/stages/readers.oci.html"
+};
 
-CREATE_SHARED_PLUGIN(1, 0, OciReader, Reader, s_info)
+CREATE_SHARED_STAGE(OciReader, s_info)
 
 std::string OciReader::getName() const { return s_info.name; }
 
@@ -68,13 +70,19 @@ void OciReader::addArgs(ProgramArgs& args)
 void OciReader::initialize()
 {
     m_compression = false;
-    m_connection = connect(m_connSpec);
+    try
+    {
+        m_connection = connect(m_connSpec);
+    }
+    catch (const connection_failed& err)
+    {
+        throwError(err.what());
+    }
     m_block = BlockPtr(new Block(m_connection));
 
     gdal::registerDrivers();
     if (m_query.empty())
-        throw pdal_error("'query' statement is empty. No data can be read "
-            "from pdal::OciReader");
+        throwError("'query' statement is empty. No data can be read.");
 
     m_stmt = Statement(m_connection->CreateStatement(m_query.c_str()));
     m_stmt->Execute(0);
@@ -86,7 +94,7 @@ void OciReader::initialize()
 
     // Fetch an initial row of data.
     if (!m_stmt->Fetch())
-        throw pdal_error("Unable to fetch a point cloud entry entry!");
+        throwError("Unable to fetch a point cloud entry entry.");
     m_block->setFetched();
 
     // If the spatial reference wasn't provided as an option, fetch it from
@@ -179,12 +187,8 @@ void OciReader::validateQuery()
     }
 
     if (!typeCorrect)
-    {
-        std::ostringstream oss;
-        oss << "Select statement '" << m_query <<
-            "' does not fetch a SDO_PC object.";
-        throw pdal_error(oss.str());
-    }
+        throwError("Select statement '" + m_query + "' does not fetch "
+            "a SDO_PC object.");
 
     // If we found all the fields, the list of required fields will be empty.
     // If not, throw an exception.
@@ -201,7 +205,7 @@ void OciReader::validateQuery()
             if (i != reqFields.end())
                oss << ",";
         }
-        throw pdal_error(oss.str());
+        throwError(oss.str());
     }
 }
 
@@ -307,36 +311,34 @@ point_count_t OciReader::readDimMajor(PointView& view, BlockPtr block,
 point_count_t OciReader::readPointMajor(PointView& view,
     BlockPtr block, point_count_t numPts)
 {
-    size_t numRemaining = block->numRemaining();
     PointId nextId = view.size();
     point_count_t numRead = 0;
 
     if (m_compression)
     {
 #ifdef PDAL_HAVE_LAZPERF
-        LazPerfBuf buf(block->chunk);
-        LazPerfDecompressor<LazPerfBuf> decompressor(buf, dbDimTypes());
-
-        std::vector<char> ptBuf(decompressor.pointSize());
-        while (numRead < numPts && numRemaining > 0)
+        auto cb = [this, &view, &nextId, &numRead](char *buf, size_t bufsize)
         {
-            point_count_t numWritten =
-                decompressor.decompress(ptBuf.data(), ptBuf.size());
-            writePoint(view, nextId, ptBuf.data());
+            writePoint(view, nextId, buf);
             if (m_cb)
                 m_cb(view, nextId);
-            numRemaining--;
             nextId++;
             numRead++;
-        }
+        };
+        const char *src = reinterpret_cast<const char *>(block->chunk.data());
+        size_t srcsize = block->chunk.size();
+        LazPerfDecompressor(cb, dbDimTypes(), block->numRemaining()).
+            decompress(src, srcsize);
 #else
-        throw pdal_error("Can't decompress without LAZperf.");
+        throwError("Can't decompress without LAZperf.");
 #endif
     }
     else
     {
         char *pos = seekPointMajor(block);
-        while (numRead < numPts && numRemaining > 0)
+
+        size_t cnt = block->numRemaining();
+        while (numRead < numPts && cnt--)
         {
             writePoint(view, nextId, pos);
 
@@ -344,12 +346,11 @@ point_count_t OciReader::readPointMajor(PointView& view,
                 m_cb(view, nextId);
 
             pos += packedPointSize();
-            numRemaining--;
             nextId++;
             numRead++;
         }
     }
-    block->setNumRemaining(numRemaining);
+    block->setNumRemaining(block->numRemaining() - numRead);
     return numRead;
 }
 
@@ -403,7 +404,7 @@ void OciReader::readBlob(Statement stmt, BlockPtr block)
 
     if (!stmt->ReadBlob(block->locator, (void*)(block->chunk.data()),
                         block->chunk.size() , &amountRead))
-        throw pdal_error("Did not read all blob data!");
+        throwError("Did not read all blob data.");
 
     block->chunk.resize(amountRead);
 }
