@@ -36,9 +36,7 @@
 #include "PgWriter.hpp"
 
 #include <pdal/PointView.hpp>
-#include <pdal/StageFactory.hpp>
 #include <pdal/XMLSchema.hpp>
-#include <pdal/pdal_macros.hpp>
 #include <pdal/util/FileUtils.hpp>
 #include <pdal/util/portable_endian.hpp>
 #include <pdal/util/ProgramArgs.hpp>
@@ -46,12 +44,14 @@
 namespace pdal
 {
 
-static PluginInfo const s_info = PluginInfo(
+static PluginInfo const s_info
+{
     "writers.pgpointcloud",
     "Write points to PostgreSQL pgpointcloud output",
-    "http://pdal.io/stages/writers.pgpointcloud.html" );
+    "http://pdal.io/stages/writers.pgpointcloud.html"
+};
 
-CREATE_SHARED_PLUGIN(1, 0, PgWriter, Writer, s_info)
+CREATE_SHARED_STAGE(PgWriter, s_info)
 
 std::string PgWriter::getName() const { return s_info.name; }
 
@@ -93,8 +93,7 @@ void PgWriter::addArgs(ProgramArgs& args)
     args.add("schema", "Schema name", m_schema_name);
     args.add("compression", "Compression type", m_compressionSpec,
         "dimensional");
-    args.add("overwrite", "Whether data should be overwritten", m_overwrite,
-        true);
+    args.add("overwrite", "Whether data should be overwritten", m_overwrite);
     args.add("srid", "SRID", m_srid, 4326U);
     args.add("pcid", "PCID", m_pcid);
     args.add("pre_sql", "SQL to execute before query", m_pre_sql);
@@ -195,26 +194,21 @@ uint32_t PgWriter::SetupSchema(uint32_t srid)
             m_pcid;
         std::string count_str = pg_query_once(m_session, oss.str());
         if (count_str.empty())
-            throw pdal_error("Unable to count pcid's in table "
-                "`pointcloud_formats`");
+            throwError("Unable to count pcid's in table `pointcloud_formats`");
         schema_count = atoi(count_str.c_str());
-        oss.str("");
         if (schema_count == 0)
-        {
-            oss << "requested PCID '" << m_pcid <<
-                "' does not exist in POINTCLOUD_FORMATS";
-            throw pdal_error(oss.str());
-        }
+            throwError("Requested PCID '" + Utils::toString(m_pcid) +
+                "' does not exist in POINTCLOUD_FORMATS");
         return m_pcid;
     }
 
     // Do we have any existing schemas in the POINTCLOUD_FORMATS table?
     uint32_t pcid = 0;
+    oss.clear();
     oss << "SELECT Count(pcid) FROM pointcloud_formats";
     std::string schema_count_str = pg_query_once(m_session, oss.str());
     if (schema_count_str.empty())
-        throw pdal_error("Unable to count pcid's in table "
-            "`pointcloud_formats`");
+        throwError("Unable to count pcid's in table 'pointcloud_formats'.");
     schema_count = atoi(schema_count_str.c_str());
     oss.str("");
 
@@ -269,7 +263,8 @@ uint32_t PgWriter::SetupSchema(uint32_t srid)
             std::string pcid_str = pg_query_once(m_session,
                     "SELECT nextval('pointcloud_formats_pcid_sq')");
             if (pcid_str.empty())
-                throw pdal_error("Unable to select nextval from pointcloud_formats_pcid_seq");
+                throwError("Unable to select nextval from "
+                    "'pointcloud_formats_pcid_seq'.");
             pcid = atoi(pcid_str.c_str());
         }
         else
@@ -284,8 +279,7 @@ uint32_t PgWriter::SetupSchema(uint32_t srid)
         std::string pcid_str = pg_query_once(m_session,
                 "SELECT Max(pcid)+1 AS pcid FROM pointcloud_formats");
         if (pcid_str.empty())
-            throw pdal_error("Unable to get the max pcid from "
-                "`pointcloud_formats`");
+            throw("Unable to get the max pcid from 'pointcloud_formats'.");
         pcid = atoi(pcid_str.c_str());
     }
 
@@ -295,7 +289,7 @@ uint32_t PgWriter::SetupSchema(uint32_t srid)
     PGresult *result = PQexecParams(m_session, oss.str().c_str(), 1,
             NULL, &paramValues, NULL, NULL, 0);
     if (PQresultStatus(result) != PGRES_COMMAND_OK)
-        throw pdal_error(PQresultErrorMessage(result));
+        throwError(PQresultErrorMessage(result));
     PQclear(result);
     m_pcid = pcid;
     return m_pcid;
@@ -312,11 +306,10 @@ void PgWriter::DeleteTable(std::string const& schema_name,
 
     if (schema_name.size())
     {
-        name << schema_name << ".";
+        name << pg_quote_identifier(schema_name) << ".";
     }
-    name << table_name;
-    stmt << pg_quote_identifier(name.str());
-
+    name << pg_quote_identifier(table_name);
+    stmt << name.str();
 
     pg_execute(m_session, stmt.str());
 }
@@ -369,7 +362,7 @@ bool PgWriter::CheckTableExists(std::string const& name)
 
     std::string count_str = pg_query_once(m_session, oss.str());
     if (count_str.empty())
-        throw pdal_error("Unable to check for the existence of `pg_table`");
+        throwError("Unable to check for the existence of 'pg_table'.");
     int count = atoi(count_str.c_str());
 
     if (count == 1)
