@@ -33,20 +33,24 @@
 ****************************************************************************/
 
 #include "SQLiteReader.hpp"
+
 #include <pdal/PointView.hpp>
-#include <pdal/pdal_macros.hpp>
+#include <pdal/pdal_features.hpp>
 #include <pdal/PDALUtils.hpp>
+#include <pdal/compression/LazPerfCompression.hpp>
 #include <pdal/util/ProgramArgs.hpp>
 
 namespace pdal
 {
 
-static PluginInfo const s_info = PluginInfo(
+static PluginInfo const s_info
+{
     "readers.sqlite",
     "Read data from SQLite3 database files.",
-    "" );
+    ""
+};
 
-CREATE_SHARED_PLUGIN(1, 0, SQLiteReader, Reader, s_info)
+CREATE_SHARED_STAGE(SQLiteReader, s_info)
 
 std::string SQLiteReader::getName() const { return s_info.name; }
 
@@ -66,17 +70,13 @@ void SQLiteReader::initialize()
         m_session->loadSpatialite(m_modulename);
 
         if (!bHaveSpatialite)
-        {
-            throw pdal_error("no spatialite enabled!");
-        }
+            throwError("Spatialite not enabled.");
 
     }
     catch (pdal_error const& e)
     {
-        std::stringstream oss;
-        oss << getName() << ": Unable to connect to database with error '" <<
-            e.what() << "'";
-        throw pdal_error(oss.str());
+        throwError("Unable to connect to database with error '" +
+            std::string(e.what()));
     }
 
     if (m_spatialRef.empty())
@@ -145,11 +145,7 @@ void SQLiteReader::validateQuery() const
     {
         auto p = m_session->columns().find(*r);
         if (p == m_session->columns().end())
-        {
-            std::ostringstream oss;
-            oss << "Unable to find required column name '" << *r << "'";
-            throw pdal_error(oss.str());
-        }
+            throwError("Unable to find required column name '" + *r + "'");
     }
 }
 
@@ -165,7 +161,7 @@ void SQLiteReader::addDimensions(PointLayoutPtr layout)
     m_session->query(q);
     const row* r = m_session->get(); // First result better have our schema
     if (!r)
-        throw pdal_error("Unable to select schema from query!");
+        throwError("Unable to select schema from query.");
 
     column const& s = r->at(0); // First column is schema
 
@@ -203,7 +199,7 @@ point_count_t SQLiteReader::readPatch(PointViewPtr view, point_count_t numPts)
 {
     const row* r = m_session->get();
     if (!r)
-        throw pdal_error("readPatch with no data in session!");
+        throwError("readPatch with no data in session.");
     std::map<std::string, int32_t> const& columns = m_session->columns();
 
     // Availability of positions already validated
@@ -214,10 +210,10 @@ point_count_t SQLiteReader::readPatch(PointViewPtr view, point_count_t numPts)
     m_patch->m_compVersion = m_patch->m_metadata.findChild("version").value();
 
     position = columns.find("NUM_POINTS")->second;
-    int32_t count;
+    size_t count;
     Utils::fromString((*r)[position].data, count);
-    m_patch->remaining = count;
-    m_patch->count = count;
+    m_patch->remaining = (int32_t)count;
+    m_patch->count = (int32_t)count;
 
     log()->get(LogLevel::Debug3) << "patch compression? "
                                  << m_patch->m_isCompressed << std::endl;
@@ -232,32 +228,36 @@ point_count_t SQLiteReader::readPatch(PointViewPtr view, point_count_t numPts)
     PointId nextId = view->size();
     if (m_patch->m_isCompressed)
     {
+        size_t bufsize = 0;
 #ifdef PDAL_HAVE_LAZPERF
-        LazPerfDecompressor<Patch> decompressor(*m_patch, dbDimTypes());
-
-        // Set the data into the patch.
-        m_patch->setBytes((*r)[position].blobBuf);
-        std::vector<char> tmpBuf(decompressor.pointSize());
-        while (numRead < numPts && count > 0)
+        auto cb = [this, view, &nextId, &numRead, &count]
+            (char *buf, size_t bufsize)
         {
-            decompressor.decompress(tmpBuf.data(), tmpBuf.size());
-            writePoint(*view.get(), nextId, tmpBuf.data());
-
+            writePoint(*view.get(),  nextId, buf);
             if (m_cb)
                 m_cb(*view, nextId);
-
             nextId++;
             numRead++;
             count--;
-        }
+        };
+
+        const char *buf = reinterpret_cast<const char *>(
+            (*r)[position].blobBuf.data());
+        bufsize = (*r)[position].blobBuf.size();
+        count = std::min(count, size_t(numPts));
+        LazPerfDecompressor(cb, dbDimTypes(), count).
+            decompress(buf, bufsize);
+
+        // Set the data into the patch.
+
 #else
-        throw pdal_error("Can't decompress without LAZperf.");
+        throwError("Can't decompress without LAZperf.");
 #endif
 
         log()->get(LogLevel::Debug3) << "Compressed byte size: " <<
-            m_patch->byte_size() << std::endl;
-        if (!m_patch->byte_size())
-            throw pdal_error("Compressed patch size was 0!");
+            bufsize << std::endl;
+        if (!bufsize)
+            throwError("Compressed patch size was 0.");
         log()->get(LogLevel::Debug3) << "Uncompressed byte size: " <<
             (m_patch->count * packedPointSize()) << std::endl;
     }
