@@ -37,10 +37,10 @@
 #include <pdal/SpatialReference.hpp>
 #include <pdal/Polygon.hpp>
 #include <pdal/util/FileUtils.hpp>
-#include <ReprojectionFilter.hpp>
-#include <MergeFilter.hpp>
-#include <LasWriter.hpp>
-#include <LasReader.hpp>
+#include <filters/ReprojectionFilter.hpp>
+#include <filters/MergeFilter.hpp>
+#include <io/LasWriter.hpp>
+#include <io/LasReader.hpp>
 
 #include <gdal_version.h>
 
@@ -48,18 +48,6 @@
 
 using namespace pdal;
 
-TEST(SpatialReferenceTest, test_env_vars)
-{
-
-#ifdef _MSC_VER
-    const char* gdal_data = getenv("GDAL_DATA");
-    const char* proj_lib = getenv("PROJ_LIB");
-
-    EXPECT_TRUE(FileUtils::fileExists(gdal_data));
-    EXPECT_TRUE(FileUtils::fileExists(proj_lib));
-#endif
-}
-
 TEST(SpatialReferenceTest, test_ctor)
 {
     SpatialReference srs;
@@ -83,22 +71,19 @@ TEST(SpatialReferenceTest, test_proj4_roundtrip)
     };
 
     {
-        SpatialReference ref;
-        ref.setProj4(proj4);
+        SpatialReference ref(proj4);
         EXPECT_TRUE(!ref.empty());
         const std::string ret = ref.getProj4();
     }
 
     {
-        SpatialReference ref;
-        ref.setProj4(proj4_ellps);
+        SpatialReference ref(proj4_ellps);
         const std::string ret = ref.getProj4();
         EXPECT_TRUE(Utils::contains(proj4_out, ret));
     }
 
     {
-        SpatialReference ref;
-        ref.setProj4(proj4_out.front());
+        SpatialReference ref(proj4_out.front());
         const std::string ret = ref.getProj4();
         EXPECT_TRUE(Utils::contains(proj4_out, ret));
     }
@@ -108,14 +93,12 @@ TEST(SpatialReferenceTest, test_proj4_roundtrip)
 // Test setting EPSG:4326 from User string
 TEST(SpatialReferenceTest, test_userstring_roundtrip)
 {
-    SpatialReference ref;
-
     std::string code = "EPSG:4326";
     std::string proj4 = "+proj=longlat +datum=WGS84 +no_defs";
     std::string proj4_ellps =
         "+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs";
     const std::string wkt = "GEOGCS[\"WGS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199433,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]]";
-    ref.setFromUserInput(code);
+    SpatialReference ref(code);
 
     std::string ret_proj = ref.getProj4();
     std::string ret_wkt = ref.getWKT();
@@ -128,11 +111,9 @@ TEST(SpatialReferenceTest, test_userstring_roundtrip)
 // Test fetching UTM zone
 TEST(SpatialReferenceTest, test_get_utmzone)
 {
-    SpatialReference ref;
-
     // from test/data/autzen-srs.wkt
     std::string code = "+proj=lcc +lat_1=43 +lat_2=45.5 +lat_0=41.75 +lon_0=-120.5 +x_0=399999.9999999999 +y_0=0 +ellps=GRS80 +units=ft +no_defs";
-    ref.setFromUserInput(code);
+    SpatialReference ref(code);
 
     BOX3D box(635589.01, 848886.45, 638994.75, 853535.43, 0, 0);
 
@@ -161,7 +142,6 @@ TEST(SpatialReferenceTest, calcZone)
 }
 
 
-#if defined(PDAL_HAVE_LIBGEOTIFF)
 // Test fetching SRS from an existing file
 TEST(SpatialReferenceTest, test_read_srs)
 {
@@ -189,10 +169,8 @@ TEST(SpatialReferenceTest, test_read_srs)
     std::string proj4 = "+proj=utm +zone=17 +datum=WGS84 +units=m +no_defs";
     EXPECT_EQ(ret_proj4, proj4);
 }
-#endif
 
 
-#ifdef PDAL_HAVE_LIBGEOTIFF
 
 // Try writing a compound coordinate system to file and ensure we get back
 // WKT with the geoidgrids (from the WKT VLR).
@@ -208,9 +186,8 @@ TEST(SpatialReferenceTest, test_vertical_datums)
 
     const std::string wkt = "COMPD_CS[\"unknown\",GEOGCS[\"WGS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"]],PRIMEM[\"Greenwich\",0],UNIT[\"degree\",0.0174532925199433],AUTHORITY[\"EPSG\",\"4326\"]],VERT_CS[\"NAVD88 height\",VERT_DATUM[\"North American Vertical Datum 1988\",2005,AUTHORITY[\"EPSG\",\"5103\"],EXTENSION[\"PROJ4_GRIDS\",\"g2012a_conus.gtx,g2012a_alaska.gtx,g2012a_guam.gtx,g2012a_hawaii.gtx,g2012a_puertorico.gtx,g2012a_samoa.gtx\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],AXIS[\"Up\",UP],AUTHORITY[\"EPSG\",\"5703\"]]]";
 
-    SpatialReference ref;
-    ref.setFromUserInput(wkt);
-    const std::string wktCheck = ref.getWKT(SpatialReference::eCompoundOK);
+    SpatialReference ref(wkt);
+    const std::string wktCheck = ref.getWKT();
     EXPECT_EQ(wkt, wktCheck); // just to make sure
 
     PointTable table;
@@ -239,7 +216,7 @@ TEST(SpatialReferenceTest, test_vertical_datums)
     reader2.execute(table2);
 
     const SpatialReference ref2 = reader2.getSpatialReference();
-    const std::string wkt2 = ref2.getWKT(SpatialReference::eCompoundOK);
+    const std::string wkt2 = ref2.getWKT();
 
     EXPECT_EQ(wkt, wkt2);
 
@@ -247,20 +224,17 @@ TEST(SpatialReferenceTest, test_vertical_datums)
     FileUtils::deleteFile(tmpfile);
 }
 **/
-#endif //PDAL_HAVE_LIBGEOTIFF
 
 
-#if defined(PDAL_HAVE_LIBGEOTIFF)
 // Try writing only the WKT VLR to a file, and see if the resulting
 // file still works ok.
 TEST(SpatialReferenceTest, test_writing_vlr)
 {
     std::string tmpfile(Support::temppath("tmp_srs_9.las"));
-    SpatialReference ref;
 
     const std::string reference_wkt = "GEOGCS[\"WGS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199433,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]]";
 
-    ref.setFromUserInput("EPSG:4326");
+    SpatialReference ref("EPSG:4326");
     std::string wkt = ref.getWKT();
     EXPECT_EQ(wkt, reference_wkt);
 
@@ -308,15 +282,13 @@ TEST(SpatialReferenceTest, test_writing_vlr)
     // Cleanup
     FileUtils::deleteFile(tmpfile);
 }
-#endif
 
 
 TEST(SpatialReferenceTest, test_io)
 {
     const std::string wkt = "COMPD_CS[\"WGS 84 + VERT_CS\",GEOGCS[\"WGS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199433,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],VERT_CS[\"NAVD88 height\",VERT_DATUM[\"North American Vertical Datum 1988\",2005,AUTHORITY[\"EPSG\",\"5103\"],EXTENSION[\"PROJ4_GRIDS\",\"g2003conus.gtx\"]],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],AXIS[\"Up\",UP],AUTHORITY[\"EPSG\",\"5703\"]]]";
 
-    SpatialReference ref;
-    ref.setFromUserInput(wkt);
+    SpatialReference ref(wkt);
 
     std::stringstream ss(std::stringstream::in | std::stringstream::out);
 
@@ -344,7 +316,6 @@ TEST(SpatialReferenceTest, test_vertical_and_horizontal)
 
 }
 
-#if defined(PDAL_HAVE_LIBGEOTIFF)
 TEST(SpatialReferenceTest, merge)
 {
     Options o1;
@@ -392,7 +363,6 @@ TEST(SpatialReferenceTest, merge)
         Support::datapath("las/test_epsg_4326x3.las"));
 }
 
-#endif
 
 TEST(SpatialReferenceTest, test_bounds)
 {
@@ -421,3 +391,17 @@ TEST(SpatialReferenceTest, test_bounds)
     EXPECT_FLOAT_EQ(static_cast<float>(b2.maxy), 39.01261687f);
 
 }
+
+// Make sure we get positive, negative and 0 back for UTM zones.
+TEST(SpatialReferenceTest, issue_1989)
+{
+    SpatialReference srs;
+    EXPECT_EQ(0, srs.getUTMZone());
+
+    SpatialReference north("EPSG:2027");
+    EXPECT_EQ(15, north.getUTMZone());
+
+    SpatialReference south("EPSG:32732");
+    EXPECT_EQ(-32, south.getUTMZone());
+}
+
