@@ -65,7 +65,7 @@ TEST(FerryFilterTest, stream)
     r.setOptions(ro);
 
     Options fo;
-    fo.add("dimensions", "X=FooX,Y=BarY");
+    fo.add("dimensions", "X=FooX,Y=>BarY");
 
     FerryFilter f;
     f.setOptions(fo);
@@ -158,6 +158,7 @@ TEST(FerryFilterTest, test_ferry_invalid)
     Options op3;
 
     op3.add("dimensions", "NewX = X");
+    op3.add("dimensions", "=>NewY");
     FerryFilter f3;
     f3.setInput(reader);
     f3.setOptions(op3);
@@ -167,11 +168,22 @@ TEST(FerryFilterTest, test_ferry_invalid)
 
     Options op4;
 
-    op4.add("dimensions", "X = Y, X = NewZ = NewQ");
+    op4.add("dimensions", "X = Y, X => NewZ = NewQ");
     FerryFilter f4;
     f4.setInput(reader);
     f4.setOptions(op4);
 
     // Make sure we reject bad option format.
     EXPECT_THROW(f4.prepare(table), pdal_error);
+
+    Options op5;
+    op5.add("dimensions", "=Foobar");
+    FerryFilter f5;
+    f5.setInput(reader);
+    f5.setOptions(op5);
+    f5.prepare(table);
+    EXPECT_TRUE(table.layout()->findDim("Foobar") != Dimension::Id::Unknown);
+    Dimension::Id id = table.layout()->findDim("NewY");
+    EXPECT_TRUE(id != Dimension::Id::Unknown);
+    EXPECT_TRUE(table.layout()->dimType(id) != Dimension::Type::None);
 }
