@@ -37,9 +37,13 @@
 #include <pdal/PointView.hpp>
 #include <pdal/StageFactory.hpp>
 #include <io/FauxReader.hpp>
+#include <io/LasReader.hpp>
+#include <io/TextReader.hpp>
 #include <filters/RangeFilter.hpp>
 #include <filters/StreamCallbackFilter.hpp>
 
+#include "Support.hpp"
+
 using namespace pdal;
 
 TEST(RangeFilterTest, createStage)
@@ -355,6 +359,31 @@ TEST(RangeFilterTest, simple_logic)
     EXPECT_EQ(view->getFieldAs<int>(Dimension::Id::X, 4), 9);
 }
 
+// Make sure that dimension names containing digits works
+TEST(RangeFilterTest, case_1659)
+{
+    TextReader reader;
+
+    Options ops;
+    ops.add("filename", Support::datapath("text/numeric_dim.txt"));
+    reader.setOptions(ops);
+
+    Options rangeOps;
+    rangeOps.add("limits", "Eigenvalue0[:35]");
+
+    RangeFilter filter;
+    filter.setOptions(rangeOps);
+    filter.setInput(reader);
+
+    PointTable table;
+    filter.prepare(table);
+    PointViewSet viewSet = filter.execute(table);
+    PointViewPtr view = *viewSet.begin();
+
+    EXPECT_EQ(1u, viewSet.size());
+    EXPECT_EQ(3u, view->size());
+}
+
 TEST(RangeFilterTest, stream_logic)
 {
     Options ops;
@@ -401,3 +430,27 @@ TEST(RangeFilterTest, stream_logic)
     f.execute(table);
 }
 
+TEST(RangeFilterTest, nan)
+{
+    LasReader reader;
+
+    Options options;
+    options.add("filename", Support::datapath("las/gps-time-nan.las"));
+    reader.setOptions(options);
+
+    Options rangeOptions;
+    rangeOptions.add("limits", "GpsTime[-1:1]");
+
+    RangeFilter filter;
+    filter.setOptions(rangeOptions);
+    filter.setInput(reader);
+
+    PointTable table;
+    filter.prepare(table);
+    PointViewSet viewSet = filter.execute(table);
+    PointViewPtr view = *viewSet.begin();
+
+    EXPECT_EQ(1u, viewSet.size());
+    EXPECT_EQ(0u, view->size());
+}
+
